package com.virjar.g4proxy_new.server.client;

import com.google.common.collect.BiMap;
import com.google.common.collect.HashBiMap;
import com.virjar.g4proxy_new.protocol.Constant;
import com.virjar.g4proxy_new.server.AvailablePortResManager;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.Channel;
import io.netty.channel.ChannelFuture;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public class ClientManager {
    private Map<String, NatClientImage> natClientImageMap = new ConcurrentHashMap<>();
    private BiMap<String, Integer> clientPortBiMap = HashBiMap.create();
    @Setter
    private ServerBootstrap userMappingBootstrap;

    public ClientManager() {
        // this.userMappingBootstrap = userMappingBootstrap;
    }

    public boolean registerNewClient(String client, Channel natChannel) {
        NatClientImage natClientImage = natClientImageMap.get(client);
        Integer mappingPort;
        if (natClientImage != null) {
            log.warn("update nat channel");
            natClientImage.getNatChannel().close();
            natClientImage.closeAllUserChannel();
            mappingPort = natClientImage.getMappingPort();

            //需要关闭所有的连接，但是计数器不能清零。计数器清零可能导致紊乱
            natClientImage.updateChannel(natChannel);
        } else {
            mappingPort = AvailablePortResManager.allocate(client);
            if (mappingPort == null) {
                log.error("failed to allocate port");
                return false;
            }
            ChannelFuture channelFuture = userMappingBootstrap.bind(mappingPort);
            try {
                channelFuture.get();
            } catch (Exception e) {
                log.error("wait for port binding error", e);
                return false;
            }
            if (!channelFuture.isSuccess()) {
                log.warn("bind mapping port:{} failed", mappingPort, channelFuture.cause());
                return false;
            }
            natClientImage = new NatClientImage(client, mappingPort, natChannel, channelFuture.channel());
        }

        natChannel.attr(Constant.NAT_CHANNEL_CLIENT_KEY).set(client);
        clientPortBiMap.put(client, mappingPort);
        natClientImageMap.put(client, natClientImage);
        log.info("client :{} register success,with port:{}", client, mappingPort);
        return true;
    }

    public NatClientImage getClient(String clientId) {
        return natClientImageMap.get(clientId);
    }

    public NatClientImage getClient(Integer mappingPort) {
        String clientId = clientPortBiMap.inverse().get(mappingPort);
        if (clientId == null) {
            return null;
        }
        return natClientImageMap.get(clientId);
    }


    public void closeClient(String clientId) {
        log.info("close client :{}", clientId);
        NatClientImage client = natClientImageMap.remove(clientId);
        if (client == null) {
            log.error("no client registered for clientId:{}", clientId);
            return;
        }
        client.closeAllUserChannel();
        client.getNatChannel().close();
        //这样，停止监听对应的代理端口，防止代理请求再次打进来
        client.getUserMappingServerChannel().close();
    }
}
