package com.virjar.nat.handlers;


import com.virjar.nat.ALOG;
import com.virjar.nat.client.ClientChannelMannager;
import com.virjar.nat.client.ProxyClient;
import com.virjar.nat.client.listener.ChannelStatusListener;
import com.virjar.nat.client.listener.ProxyChannelBorrowListener;
import com.virjar.nat.protocol.Constants;
import com.virjar.nat.protocol.ProxyMessage;

import java.net.InetSocketAddress;

import io.netty.bootstrap.Bootstrap;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import io.netty.channel.Channel;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelFutureListener;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelOption;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.handler.timeout.IdleState;
import io.netty.handler.timeout.IdleStateEvent;


/**
 * 代理客户端
 *
 * @author ynz
 * @version 创建时间：2018年6月27日 下午6:13:03
 * @email ynz@myframe.cn
 */
public class ClientChannelHandler extends SimpleChannelInboundHandler<ProxyMessage> {


    private Bootstrap join2LittleProxyBootStrap;

    private Bootstrap join2NatServerBootStrap;

    private ChannelStatusListener channelStatusListener;

    private String keys;

    private ProxyClient proxyClient;

    public ClientChannelHandler(Bootstrap join2LittleProxyBootStrap, Bootstrap join2NatServerBootStrap, ChannelStatusListener channelStatusListener, ProxyClient proxyClient, String keys) {
        this.join2LittleProxyBootStrap = join2LittleProxyBootStrap;
        this.join2NatServerBootStrap = join2NatServerBootStrap;
        this.channelStatusListener = channelStatusListener;
        this.keys = keys;
        this.proxyClient = proxyClient;
    }


    @Override
    protected void channelRead0(ChannelHandlerContext ctx, ProxyMessage proxyMessage) throws Exception {
        ALOG.i(Constants.tag, "recieved proxy message, type is:" + proxyMessage.getTypeReadable());
        switch (proxyMessage.getType()) {
            case ProxyMessage.TYPE_CONNECT:
                handleConnectMessage(ctx, proxyMessage);
                break;
            case ProxyMessage.TYPE_DISCONNECT:
                handleDisconnectMessage(ctx, proxyMessage);
                break;
            case ProxyMessage.P_TYPE_TRANSFER:
                handleTransferMessage(ctx, proxyMessage);
                break;
            default:
                break;
        }
    }

    private void handleTransferMessage(ChannelHandlerContext ctx, ProxyMessage proxyMessage) {
        Channel littleProxyServerChannel = ctx.channel().attr(Constants.NEXT_CHANNEL).get();
        if (littleProxyServerChannel == null) {
            //现在这个不会发生了
            ALOG.w(Constants.tag, "littleProxy Connection lost");
            proxyClient.getClientChannelMannager().returnProxyChanel(ctx.channel());
            return;
        }
        ByteBuf buf = ctx.alloc().buffer(proxyMessage.getData().length);
        buf.writeBytes(proxyMessage.getData());
        ALOG.i(Constants.tag, "write data to littleProxy server, " + littleProxyServerChannel);
        littleProxyServerChannel.writeAndFlush(buf);
    }

    private void handleDisconnectMessage(ChannelHandlerContext ctx, ProxyMessage proxyMessage) {
        Channel littleProxyServerChannel = ctx.channel().attr(Constants.NEXT_CHANNEL).get();
        ALOG.i(Constants.tag, "handleDisconnectMessage, :" + littleProxyServerChannel);
        if (littleProxyServerChannel != null) {
            ctx.channel().attr(Constants.NEXT_CHANNEL).remove();
            proxyClient.getClientChannelMannager().returnProxyChanel(ctx.channel());
            littleProxyServerChannel.writeAndFlush(Unpooled.EMPTY_BUFFER).addListener(ChannelFutureListener.CLOSE);
        }
    }

    private void handleConnectMessage(ChannelHandlerContext ctx, ProxyMessage proxyMessage) {
        final Channel cmdChannel = ctx.channel();
        final String userId = proxyMessage.getUri();

        final InetSocketAddress sa = (InetSocketAddress) ctx.channel().remoteAddress();
        join2LittleProxyBootStrap.connect("127.0.0.1", 3128).addListener(new ChannelFutureListener() {

            @Override
            public void operationComplete(ChannelFuture future) throws Exception {

                // 连接后端服务器成功
                if (future.isSuccess()) {
                    final Channel littleProxyServerChannel = future.channel();
                    ALOG.i(Constants.tag, "connect to little proxy server success, :" + littleProxyServerChannel);
                    littleProxyServerChannel.config().setOption(ChannelOption.AUTO_READ, false);

                    // 获取连接
                    proxyClient.getClientChannelMannager().borrowProxyChanel(sa.getPort(), join2NatServerBootStrap, new ProxyChannelBorrowListener() {

                        @Override
                        public void success(Channel natServerChannel) {
                            // 连接绑定.attr(Constants.NEXT_CHANNEL).set(littleProxyServerChannel);
                            littleProxyServerChannel.attr(Constants.NEXT_CHANNEL).set(natServerChannel);
                            natServerChannel.attr(Constants.NEXT_CHANNEL).set(littleProxyServerChannel);

                            // 远程绑定
                            ProxyMessage proxyMessage = new ProxyMessage();
                            proxyMessage.setType(ProxyMessage.TYPE_CONNECT);
                            proxyMessage.setUri(userId + "@" + keys);
                            natServerChannel.writeAndFlush(proxyMessage);

                            littleProxyServerChannel.config().setOption(ChannelOption.AUTO_READ, true);
                            proxyClient.getClientChannelMannager().addLittleProxyServerChannel(userId, littleProxyServerChannel);
                            proxyClient.getClientChannelMannager().setRealServerChannelUserId(littleProxyServerChannel, userId);
                        }

                        @Override
                        public void error(Throwable cause) {
                            ProxyMessage proxyMessage = new ProxyMessage();
                            proxyMessage.setType(ProxyMessage.TYPE_DISCONNECT);
                            proxyMessage.setUri(userId);
                            cmdChannel.writeAndFlush(proxyMessage);
                        }
                    });

                } else {
                    ProxyMessage proxyMessage = new ProxyMessage();
                    proxyMessage.setType(ProxyMessage.TYPE_DISCONNECT);
                    proxyMessage.setUri(userId);
                    cmdChannel.writeAndFlush(proxyMessage);
                }
            }
        });
    }


    /**
     * 事件触发
     *
     * @param ctx
     * @param evt
     * @throws Exception
     */
    @Override
    public void userEventTriggered(ChannelHandlerContext ctx, Object evt) throws Exception {
        if (evt instanceof IdleStateEvent) {
            IdleStateEvent idleStateEvent = (IdleStateEvent) evt;
            if (idleStateEvent.state() == IdleState.WRITER_IDLE) {
                ALOG.w(Constants.tag, "已经 10 秒没有发送信息！");
                //向服务端发送消息

            }
        }
        super.userEventTriggered(ctx, evt);
    }

    @Override
    public void channelWritabilityChanged(ChannelHandlerContext ctx) throws Exception {
        Channel realServerChannel = ctx.channel().attr(Constants.NEXT_CHANNEL).get();
        if (realServerChannel != null) {
            realServerChannel.config().setOption(ChannelOption.AUTO_READ, ctx.channel().isWritable());
        }
        super.channelWritabilityChanged(ctx);
    }

    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {

        // 控制连接
        if ( proxyClient.getClientChannelMannager().getCmdChannel() == ctx.channel()) {
            proxyClient.getClientChannelMannager().setCmdChannel(null);
            proxyClient.getClientChannelMannager().clearRealServerChannels();
            channelStatusListener.channelInactive(ctx);
        } else {
            // 数据传输连接
            Channel realServerChannel = ctx.channel().attr(Constants.NEXT_CHANNEL).get();
            if (realServerChannel != null && realServerChannel.isActive()) {
                realServerChannel.close();
            }
        }

        proxyClient.getClientChannelMannager().removeProxyChanel(ctx.channel());
        super.channelInactive(ctx);
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        ALOG.e(Constants.tag, "exception caught", cause);
        super.exceptionCaught(ctx, cause);
    }

}