package com.virjar.nat.handlers;

import com.virjar.nat.ALOG;
import com.virjar.nat.client.ClientChannelMannager;
import com.virjar.nat.client.ProxyClient;
import com.virjar.nat.protocol.Constants;
import com.virjar.nat.protocol.ProxyMessage;

import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelOption;
import io.netty.channel.SimpleChannelInboundHandler;

/**
 * 真实请求channel.
 */
public class LittleProxyServerChannelHandler extends SimpleChannelInboundHandler<ByteBuf> {

    private ProxyClient proxyClient;

    public LittleProxyServerChannelHandler(ProxyClient proxyClient) {
        this.proxyClient = proxyClient;
    }

    @Override
    protected void channelRead0(ChannelHandlerContext ctx, ByteBuf buf) throws Exception {
        Channel littleProxyChannel = ctx.channel();
        Channel natDataChannel = littleProxyChannel.attr(Constants.NEXT_CHANNEL).get();
        if (natDataChannel == null) {
            // 代理客户端连接断开
            ctx.channel().close();
        } else {
            byte[] bytes = new byte[buf.readableBytes()];
            buf.readBytes(bytes);
            String userId = proxyClient.getClientChannelMannager().getRealServerChannelUserId(littleProxyChannel);
            ProxyMessage proxyMessage = new ProxyMessage();
            proxyMessage.setType(ProxyMessage.P_TYPE_TRANSFER);
            proxyMessage.setUri(userId);
            proxyMessage.setData(bytes);
            natDataChannel.writeAndFlush(proxyMessage);
            ALOG.i(Constants.tag, "write data to nat server, littleProxyChannel:" + littleProxyChannel + "  natDataChannel " + natDataChannel);
        }
    }

    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        super.channelActive(ctx);
    }

    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        Channel realServerChannel = ctx.channel();
        String userId = proxyClient.getClientChannelMannager().getRealServerChannelUserId(realServerChannel);
        proxyClient.getClientChannelMannager().removeRealServerChannel(userId);
        Channel channel = realServerChannel.attr(Constants.NEXT_CHANNEL).get();
        if (channel != null) {
            ALOG.i(Constants.tag, "channelInactive :" + realServerChannel);
            ProxyMessage proxyMessage = new ProxyMessage();
            proxyMessage.setType(ProxyMessage.TYPE_DISCONNECT);
            proxyMessage.setUri(userId);
            channel.writeAndFlush(proxyMessage);
        }

        super.channelInactive(ctx);
    }

    @Override
    public void channelWritabilityChanged(ChannelHandlerContext ctx) throws Exception {
        Channel realServerChannel = ctx.channel();
        Channel proxyChannel = realServerChannel.attr(Constants.NEXT_CHANNEL).get();
        if (proxyChannel != null) {
            proxyChannel.config().setOption(ChannelOption.AUTO_READ, realServerChannel.isWritable());
        }

        super.channelWritabilityChanged(ctx);
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        ALOG.e(Constants.tag, "exception caught", cause);
        super.exceptionCaught(ctx, cause);
    }
}